require 'time'
require 'date'

class ViewTerm
  def initialize(hash)
    @start_year = hash[:start_year].to_i
    @start_mon = hash[:start_mon].to_i
    @end_year = hash[:end_year].to_i
    @end_mon = hash[:end_mon].to_i
    @ctr = hash[:ctr].to_i

    if Date.exist?(@start_year, @start_mon, 1)
      @sd = Date.new(@start_year, @start_mon, 1)
    end
    if Date.exist?(@end_year, @end_mon, 1)
      @ed = (Date.new(@end_year, @end_mon, 1) >> 1) - 1
    end
  end
  attr_reader :start_year, :start_mon, :end_year, :end_mon
  attr_reader :sd, :ed, :ctr

  def errors
    ActiveRecord::Errors.new(self)
  end
end

# 総勘定元帳
class GeneralLedgerController < ApplicationController
  before_filter :login_required
  
  def index
  end
  
  # 部門の予算・実績対比表の出力
  def comparison
    @cdiv = current_user.division.default_controller
    raise if !@cdiv
=begin
SELECT * FROM general_ledger WHERE (date BETWEEN ? AND ?) AND
     division_id IN (SELECT division_id FROM controllers_divisions
                    WHERE control_division_id = ?)
=end
  end
  
  def view
  end
  
  def view2
    @term = ViewTerm.new(params[:term])
    @controllers = ControlDivision.find :all, :conditions => ['parent_id = ?', @term.ctr]
  end
  
  # 実績データの削除
  def clear
    term = ViewTerm.new(params[:term])
    raise if !term.sd || !term.ed || term.sd > term.ed
    
    actual_kubun = Kubun.find_by_name('actual')
    GeneralLedger.delete_all ['kubun_id = ? and date between ? and ?',
                              actual_kubun.id, term.sd, term.ed]
  end
  
  # 科目別元帳
  def item_ledger
    term = ViewTerm.new(params[:term])
    raise if !term.sd || !term.ed || term.sd > term.ed
    
    actual_kubun = Kubun.find_by_name('actual')
    item = AccountTitle.find(params[:item][:id])
    @gl = GeneralLedger.find(:all,
          :conditions => ['kubun_id = ? and (date between ? and ?) and account_title_id = ?',
             actual_kubun.id, term.sd, term.ed, item.id],
          :include => :account_title)
    
    # 出力
    headers["Content-Type"] = "application/vnd.ms-excel; charset=UTF-8"
    headers["Content-Disposition"] = "attachment;\n  filename*=iso-2022-jp'ja'" +
                                     NKF.nkf("-m0 -j -W", "科目別元帳.xls")
    render :action => 'item_ledger', :layout => false
  end
  

  def download
    control_division = ControlDivision.find(params[:division]) if params[:division]
    root = ControlDivision.find params[:root]
    sd = Date.new(params[:sy].to_i, params[:sm].to_i, 1)
    ed = (Date.new(params[:ey].to_i, params[:em].to_i, 1) >> 1) - 1
    raise if sd > ed
    
    if control_division
      # 部門別
      case params[:kubun]
      when 'actual', 'estimate'
        actual_kubun = Kubun.find_by_name(params[:kubun])
        cond = ["kubun_id = ? AND control_division_id = ? AND (date >= ? AND date <= ?)",
              actual_kubun.id, control_division.id, sd, ed]
      when 'taihi'
        cond = ["control_division_id = ? AND (date >= ? AND date <= ?)",
                control_division.id, sd, ed]
      else
        raise
      end
      gl = GeneralLedger.find(:all,
               :select => "general_ledger.*",
               :joins => "INNER JOIN divisions ON general_ledger.division_id = divisions.id",
               :conditions => cond)
    else
      # 全社計
      case params[:kubun]
      when 'actual', 'estimate'
        actual_kubun = Kubun.find_by_name(params[:kubun])
        cond = ["kubun_id = ? AND date >= ? AND date <= ?", actual_kubun.id, sd, ed]
        gl = GeneralLedger.find(:all, :conditions => cond)
      when 'taihi'
        cond = ["(division_id IS NOT NULL) AND date >= ? AND date <= ?", sd, ed]
        gl = GeneralLedger.find(:all,
                :select => 'date, kubun_id, division_id, account_title_id, sum(amount) as amount',
                :conditions => cond,
                :group => 'date, kubun_id, division_id, account_title_id')
      else
        raise
      end
    end

    data = <<EOF
<html xmlns:o="urn:schemas-microsoft-com:office:office"
      xmlns:x="urn:schemas-microsoft-com:office:excel"
      xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja"<
<head>
  <meta http-equiv='Content-Type' content='text/html;charset=utf-8'>
  <meta name='ProgId' content='Excel.Sheet'>
  <xml>
    <o:DocumentProperties>
      <o:Created>#{Time.now.iso8601}</o:Created>
      <o:Company>株式会社スターフライヤー</o:Company>
    </o:DocumentProperties>
  </xml>
  <style>
    @page {
        margin:.59in .39in .39in .39in;
        mso-header-margin:.51in;
        mso-footer-margin:.31in;
        mso-page-orientation:landscape;}
    td {
      font-size:9.0pt; }
  </style>
</head>
<body>
    <h1>#{control_division ? control_division.name : nil} #{sd}-#{ed} 実績</h1>
<table border="0" cellpadding="0" cellspacing="0" width="100%"
    style='border-collapse:collapse;table-layout:fixed;'>
<tr>
  <th>伝票日付</th>
  <th>年月</th>
  <th>区分</th>
  <th>伝票NO</th>
  <th>作成日</th>
  <th>作成者コード</th>
  <th>作成者名</th>
  <th>科目コード</th>
  <th>補助コード</th>
  <th>科目名</th>
  <th>科目名2</th>
  <th>部門コード</th>
  <th>部門名</th>
  <th>金額</th>
  <th>摘要</th>
</tr>
EOF
    not_output = [722, 801, 802, 803, 804, 805, 806, 807, 808, 809, 810, 831, 832, 833, 834, 835, 836, 837, 838, 905, 913, 915]
    # TODO: 決めうちしてる。fixme
    
    gl.each {|r|
      # 科目の抽出
      if control_division
        next if not_output.include?(r.account_title.account_code) ||
                r.account_title.account_code < 601
      elsif params[:kubun] == 'taihi'
        next if r.account_title.account_code < 601
      end

      # TODO: kubun_id is magic.
      data << <<EOF
<tr>
  <td>#{r.date}</td>
  <td>#{sprintf("%04d%02d", r.date.year, r.date.mon)}</td>
  <td>#{Kubun.find(r.kubun_id).long_name}</td>
  <td>#{r.has_attribute?('slip_number') ? r.slip_number : ''}</td>
  <td>#{r.has_attribute?('created_at') && r.created_at ? r.created_at.strftime("%Y/%m/%d") : ''}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.worker_code : nil}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.name : nil}</td>
  <td>#{r.account_title.account_code}</td>
  <td>#{r.account_title.suppl_code}</td>
  <td>#{r.account_title.name}</td>
  <td>#{r.account_title.account_code}#{r.account_title.main_name}
  <td>#{control_division ? r.division.division_code : nil}</td>
  <td>#{control_division ? r.division.name :
            (r.division ? r.division.controller(root).name : '')}</td>
  <td>#{r.amount}</td>
  <td>#{r.has_attribute?('remarks') ? r.remarks : ''}</td>
</tr>
EOF
    }
    data << "</table></body></html>"
    
    send_data data, :type => "application/vnd.ms-excel; charset=UTF-8", :disposition => %Q(attachment; filename="#{
    sprintf("%04d%02d", ed.year, ed.mon)}#{control_division ? control_division.name : 'all'}.xls")
  end
end
